unit Unit1;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  System.StrUtils,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  IdWebDAV, IdURI;

type
  TGetThread = class(TThread)
  private
    FURL, FFileName: String;
    FRunning: Boolean;
    FReply: String;
    FResult: Boolean;
    procedure DoAfterSync;
  protected
    sErrMsg: string;
    procedure Execute ; override;
  public
    constructor Create(URL, FileName: String);
    property Running: Boolean read FRunning;
    function GetReply(var Reply: String): Boolean;
  end;

  TThreadMain = class(TThread)
  private
    procedure Execute; override;
  end;

  TForm1 = class(TForm)
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.fmx}

function DAVGetFile(Source: String; Target: String): String;
var
  URL: String;
  IdWebDAV: TIdWebDAV;
  Stream: TFileStream;
begin
  Result := '';
  try
    IdWebDAV := TIdWebDAV.Create(nil);
    IdWebDAV.Request.BasicAuthentication := True;
    IdWebDAV.Request.Username := 'User';
    IdWebDAV.Request.Password := 'Password';
    IdWebDAV.ReadTimeout := 10000;
    IdWebDAV.Request.CacheControl := 'no-cache';
    URL := TIdUri.URLEncode(Source);
    try
      Stream := TFileStream.Create(Target, fmCreate or fmOpenWrite);
      IdWebDAV.Get(URL, Stream);
      FreeAndNil(Stream);
    except
      on e:exception do
        Result := e.message;
    end;
  finally
    FreeAndNil(IdWebDAV);
  end;
end;

constructor TGetThread.Create(URL, FileName: String);
begin
  inherited Create(True);
  FURL := URL;
  FFileName := FileName;
  FRunning := True;
  FReply := '';
  Resume;
end;

procedure TGetThread.Execute;
var
  Text: string;
begin
  FRunning := true;
  sErrMsg := DAVGetFile(FURL, FFileName);
  FResult := sErrMsg = '';
  FReply := ifThen(sErrMsg <> '', sErrMsg, '');
  Synchronize(DoAfterSync);
end;

function TGetThread.GetReply(var Reply: String): Boolean;
begin
  Reply := FReply;
  Result := FResult;
end;

procedure TGetThread.DoAfterSync;
begin
  FRunning := False;
end;

procedure TThreadMain.Execute;
var
  GetThread: TGetThread;
begin
  GetThread := TGetThread.Create('http://10.1.1.1:8080/ABC.zip', 'C:\Temp\ABC.zip');
  while GetThread.Running do Application.ProcessMessages;
  ShowMessage('Completed');
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  ThreadMain: TThreadMain;
begin
  ThreadMain := TThreadMain.Create(false);
end;

end.
